import React from 'react';
import PropTypes from 'prop-types';
import moment from 'moment-timezone';
import 'react-fontawesome';
import WebFont from 'webfontloader';
import Countdown from './Countdown';
import Drawing from './Drawing';
import Header from './Header';
import Footer from './Footer';
import getUpcomingDate from './getUpcomingDate';

import style from './App.sass';

WebFont.load({
  google: {
    families: [
      'Titillium Web:300,400,700',
    ],
  },
});

class App extends React.Component {
  static propTypes = {
    getNow: PropTypes.func,
    weekdays: PropTypes.arrayOf(
      PropTypes.oneOfType([
        PropTypes.string,
        PropTypes.number,
      ])),
  }

  static defaultProps = {
    getNow: () => moment(),
    weekdays: [
      2,
      4,
      6,
    ],
  }

  componentDidMount() {
    this.interval = setInterval(this.updateUpcomingDate.bind(this), 1000);
    this.updateUpcomingDate();
  }

  componentWillUnmount() {
    clearInterval(this.interval);
    this.interval = null;
  }

  calculateUpcomingDate() {
    const { getNow, weekdays } = this.props;
    const now = getNow();
    return moment.min(
      ...weekdays.map(weekday => getUpcomingDate(now, weekday)),
    );
  }

  updateUpcomingDate() {
    const { getNow } = this.props;

    const nextUpcomingDate = this.calculateUpcomingDate();

    if (this.interval) {
      const interval = moment.duration(nextUpcomingDate.diff(getNow()))
        .asMilliseconds();

      clearInterval(this.interval);
      this.interval = setInterval(this.updateUpcomingDate.bind(this), interval);
    }

    this.setState({ nextUpcomingDate });
  }

  render() {
    if (!this.state) {
      return null;
    }

    const { nextUpcomingDate } = this.state;
    if (!nextUpcomingDate) {
      return null;
    }

    return (
      <div className={style.app}>
        <Header>
          <b>rizon</b> VIzon countdown
        </Header>

        <div className={style.content}>
          <p>
            The next VIzon draw is on {nextUpcomingDate.format('dddd')}, {nextUpcomingDate.format('L LT z')}.
          </p>
          <Countdown date={nextUpcomingDate} />

          <p>
              Last Drawing:
          </p>
          <Drawing
            numbers={[1, 2, 3, 4, 5, 6]}
            ranking={[
              { rank: '1st', winners: 0 },
              { rank: '2nd', winners: 0 },
              { rank: '3rd', winners: 0 },
              { rank: 'Consolation', winners: 4 },
            ]}
          />
        </div>

        <Footer>
          Made by Icedream & MAGIC
        </Footer>
      </div>
    );
  }
}

export default App;
